==> heteroscalapack-1.0.2-BETA.tar.gz: heteroscalapack-1.0.2-BETA/tests/PBLAS/level3/test_hscal_pztranc_auto.c <==

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>

#include <hscalapack.h>

#include "hscalapack_pztranc_parameters.h"

int main(int argc, char **argv)
{
    int        DESCA[DLEN_], DESCC[DLEN_];
    int        i, rc;
    int        ictxt, myrow, mycol;
    int        Ka, Kc;
    int        CHECK_RESULT = 0;
    int     c__0    = 0;

    int        p, q;
    HMPI_Group gid;
    MPI_Comm   pztranccomm;

    hscaldoublecomplex    *A, *C;
 
    struct timeval start, end;
    struct timeval rstart, rend;
    struct timeval gstart, gend;
    struct timeval astart, aend;

    gettimeofday(&start, NULL);

    //
    // Initialize HMPI runtime
    HMPI_Init(&argc, &argv);

    HMPI_Barrier(HMPI_COMM_WORLD_GROUP);

    DESCA[DTYPE_] = BLOCK_CYCLIC_2D_INB;
    DESCA[M_]     = N;
    DESCA[N_]     = M;
    DESCA[IMB_]   = NB;
    DESCA[INB_]   = MB;
    DESCA[MB_]    = NB;
    DESCA[NB_]    = MB;
    DESCA[RSRC_]  = 0;
    DESCA[CSRC_]  = 0;

    DESCC[DTYPE_] = BLOCK_CYCLIC_2D_INB;
    DESCC[M_]     = M;
    DESCC[N_]     = N;
    DESCC[IMB_]   = MB;
    DESCC[INB_]   = NB;
    DESCC[MB_]    = MB;
    DESCC[NB_]    = NB;
    DESCC[RSRC_]  = 0;
    DESCC[CSRC_]  = 0;

    if (HMPI_Is_host())
    {
       gettimeofday(&rstart, NULL);
    }

    rc = hscal_pztranc_recon(
              &M,
              &N,
              (double*)&ALPHA,
              &IA,
              &JA,
              DESCA,
              (double*)&BETA,
              &IC,
              &JC,
              DESCC
    );

    if (rc != HMPI_SUCCESS)
    {
       return rc;
    }

    HMPI_Barrier(HMPI_COMM_WORLD_GROUP);

    if (HMPI_Is_host())
    {
       gettimeofday(&rend, NULL);
    }

    if (HMPI_Is_host())
    {
       gettimeofday(&gstart, NULL);
    }

    HMPI_Debug(1);

    rc = hscal_pztranc_group_heuristic_auto_create(
              &M,
              &N,
              (double*)&ALPHA,
              &IA,
              &JA,
              DESCA,
              (double*)&BETA,
              &IC,
              &JC,
              DESCC,
              &gid,
              &MPC_NetType_pztranc,
              hscal_heuristic_fn_all
    );

    if (rc != HSCAL_SUCCESS)
    {
       return rc;
    }

    HMPI_Debug(0);

    if (HMPI_Is_host())
    {
       gettimeofday(&gend, NULL);
    }

    if (HMPI_Is_free())
    {
       HMPI_Finalize(0);
    }

    HMPI_Barrier(&gid);

    if (HMPI_Is_host())
    {
       gettimeofday(&astart, NULL);
    }

    pztranccomm = *(MPI_Comm*)HMPI_Get_comm(&gid);

    /*
     * Broadcast the number of processes to
     * all the processors
     */
    if (HMPI_Is_parent(&gid))
    {
       int nd;
       int *d;

       HMPI_Group_topology(
           &gid,
           &nd,
           &d
       );

       p = d[0];

       free(d);

       rc = MPI_Bcast(
               &p,
               1,
               MPI_INT,
               0,
               pztranccomm
       );
    }
    else
    {
       rc = MPI_Bcast(
               &p,
               1,
               MPI_INT,
               0,
               pztranccomm
       );
    }

    if (rc != MPI_SUCCESS)
    {
       printf("Problems broadcasting p\n");
       MPI_Abort(MPI_COMM_WORLD, -9);
    }

    if (HMPI_Is_parent(&gid))
    {
       int nd;
       int *d;

       HMPI_Group_topology(
           &gid,
           &nd,
           &d
       );

       q = d[1];

       free(d);

       rc = MPI_Bcast(
               &q,
               1,
               MPI_INT,
               0,
               pztranccomm
       );
    }
    else
    {
       rc = MPI_Bcast(
               &q,
               1,
               MPI_INT,
               0,
               pztranccomm
       );
    }

    if (rc != MPI_SUCCESS)
    {
       printf("Problems broadcasting q\n");
       MPI_Abort(MPI_COMM_WORLD, -9);
    }

    if (HMPI_Is_parent(&gid))
    {
       printf("Optimal p=%d, Optimal q=%d\n", p, q);
    }

    /*
     * Translate algocomm to a BLACS handle
     */
    ictxt = Csys2blacs_handle(pztranccomm);

    /*
     * Form BLACS context based on algocomm
     */
    Cblacs_gridinit(&ictxt, "r", p, q);
    Cblacs_gridinfo(ictxt, &p, &q, &myrow, &mycol);

    DESCA[DTYPE_] = BLOCK_CYCLIC_2D_INB;
    DESCA[CTXT_]  = ictxt;
    DESCA[M_]     = N;
    DESCA[N_]     = M;
    DESCA[IMB_]   = NB;
    DESCA[INB_]   = MB;
    DESCA[MB_]    = NB;
    DESCA[NB_]    = MB;
    DESCA[RSRC_]  = 0;
    DESCA[CSRC_]  = 0;

    DESCC[DTYPE_] = BLOCK_CYCLIC_2D_INB;
    DESCC[CTXT_]  = ictxt;
    DESCC[M_]     = M;
    DESCC[N_]     = N;
    DESCC[IMB_]   = MB;
    DESCC[INB_]   = NB;
    DESCC[MB_]    = MB;
    DESCC[NB_]    = NB;
    DESCC[RSRC_]  = 0;
    DESCC[CSRC_]  = 0;

    DESCA[LLD_]   = max(1, PB_Cnumroc(DESCA[M_],
                                      0,
                                      DESCA[IMB_],
                                      DESCA[MB_],
                                      myrow,
                                      DESCA[RSRC_],
                                      p
                       )
    );

    Ka            = max(1, PB_Cnumroc(JA+M-1,
                                      0,
                                      DESCA[INB_],
                                      DESCA[NB_],
                                      mycol,
                                      DESCA[CSRC_],
                                      q
                       )
    );

    DESCC[LLD_]   = max(1, PB_Cnumroc(DESCC[M_],
                                      0,
                                      DESCC[IMB_],
                                      DESCC[MB_],
                                      myrow,
                                      DESCC[RSRC_],
                                      p
                       )
    );

    Kc            = max(1, PB_Cnumroc(JC+N-1,
                                      0,
                                      DESCC[INB_],
                                      DESCC[NB_],
                                      mycol,
                                      DESCC[CSRC_],
                                      q
                       )
    );

    A = (hscaldoublecomplex*)malloc(
                  sizeof(hscaldoublecomplex)
                  *
                  (DESCA[LLD_]*Ka)
    );

    if (A == NULL)
    {
       printf("Cannot allocate A\n");
       HMPI_Finalize(-1);
    }

    C = (hscaldoublecomplex*)malloc(
                  sizeof(hscaldoublecomplex)
                  *
                  (DESCC[LLD_]*Kc)
    );

    if (C == NULL)
    {
       printf("Cannot allocate C\n");
       HMPI_Finalize(-1);
    }

    {
       int     iaseed = 100;
       hscalt_pzmatgen(&ictxt, "N", "N", &DESCA[M_],
                 &DESCA[N_], &DESCA[MB_], &DESCA[NB_], A, &DESCA[LLD_],
                 &DESCA[RSRC_], &DESCA[CSRC_], &iaseed, &c__0, &DESCA[LLD_], &c__0, &Ka,
                 &myrow, &mycol, &p, &q);
    }

    {
       int     iaseed = 200;
       hscalt_pzmatgen(&ictxt, "N", "N", &DESCC[M_],
                 &DESCC[N_], &DESCC[MB_], &DESCC[NB_], C, &DESCC[LLD_],
                 &DESCC[RSRC_], &DESCC[CSRC_], &iaseed, &c__0, &DESCC[LLD_], &c__0, &Kc,
                 &myrow, &mycol, &p, &q);
    }

    pztranc_(
      &M, &N, 
      (double*)&ALPHA,
      (double*)A, &IA, &JA, DESCA,
      (double*)&BETA,
      (double*)C, &IC, &JC, DESCC
    );

    //
    // Check the results
    if (CHECK_RESULT)
    {
       if (HMPI_Is_host())
       {
          for (i = 0; i < (DESCC[LLD_]*Kc); i++)
          {
              printf("C[%d]=(%f, %f) ", i, C[i].r, C[i].i);
          }
          printf("\n");
       }
    }

    free(A);
    free(C);

    Cblacs_gridexit(ictxt);

    HMPI_Barrier(&gid);

    if (HMPI_Is_member(&gid))
    {
       HMPI_Group_free(
           &gid
       );
    }

    if (HMPI_Is_host())
    {
       gettimeofday(&aend, NULL);
    }

    if (HMPI_Is_host())
    {
       gettimeofday(&end, NULL);
    }

    /*
     * Print Execution time
     */
    if (HMPI_Is_host())
    {
       double rtstart = rstart.tv_sec + rstart.tv_usec/1000000.;
       double rtend = rend.tv_sec + rend.tv_usec/1000000.;
       double rt_sec = rtend - rtstart;

       double gtstart = gstart.tv_sec + gstart.tv_usec/1000000.;
       double gtend = gend.tv_sec + gend.tv_usec/1000000.;
       double gt_sec = gtend - gtstart;

       double atstart = astart.tv_sec + astart.tv_usec/1000000.;
       double atend = aend.tv_sec + aend.tv_usec/1000000.;
       double at_sec = atend - atstart;

       double tstart = start.tv_sec + start.tv_usec/1000000.;
       double tend   = end.tv_sec + end.tv_usec/1000000.;
       double t_sec = tend - tstart;

       printf(
          "M=%d, N=%d, MB=%d, NB=%d, "
          "Recon time(sec)=%f, "
          "Group creation time(sec)=%f, Algorithm execution time (sec)=%f, total time(sec)=%f\n",
          M,
          N,
          MB,
          NB,
          rt_sec,
          gt_sec,
          at_sec,
          t_sec
       );
    }

    HMPI_Finalize(0);
}

==> heteroscalapack-1.0.7-BETA.tar.gz: heteroscalapack-1.0.7-BETA/tests/PBLAS/level3/test_hscal_pztranc_auto.c <==

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>

#include <hscalapack.h>

#include "hscalapack_pztranc_parameters.h"

int main(int argc, char **argv)
{
    int        DESCA[DLEN_], DESCC[DLEN_];
    int        i, rc;
    int        ictxt, myrow, mycol;
    int        Ka, Kc;
    int        CHECK_RESULT = 0;
    int     c__0    = 0;

    int        p, q;
    HMPI_Group gid;
    MPI_Comm   pztranccomm;

    hscaldoublecomplex    *A, *C;
 
    struct timeval start, end;
    struct timeval rstart, rend;
    struct timeval gstart, gend;
    struct timeval astart, aend;

    gettimeofday(&start, NULL);

    //
    // Initialize HMPI runtime
    HMPI_Init(&argc, &argv);

    DESCA[DTYPE_] = BLOCK_CYCLIC_2D_INB;
    DESCA[M_]     = N;
    DESCA[N_]     = M;
    DESCA[IMB_]   = NB;
    DESCA[INB_]   = MB;
    DESCA[MB_]    = NB;
    DESCA[NB_]    = MB;
    DESCA[RSRC_]  = 0;
    DESCA[CSRC_]  = 0;

    DESCC[DTYPE_] = BLOCK_CYCLIC_2D_INB;
    DESCC[M_]     = M;
    DESCC[N_]     = N;
    DESCC[IMB_]   = MB;
    DESCC[INB_]   = NB;
    DESCC[MB_]    = MB;
    DESCC[NB_]    = NB;
    DESCC[RSRC_]  = 0;
    DESCC[CSRC_]  = 0;

    if (HMPI_Is_host())
    {
       gettimeofday(&rstart, NULL);
    }

    rc = hscal_pztranc_recon(
              &M,
              &N,
              (double*)&ALPHA,
              &IA,
              &JA,
              DESCA,
              (double*)&BETA,
              &IC,
              &JC,
              DESCC
    );

    if (rc != HMPI_SUCCESS)
    {
       return rc;
    }

    if (HMPI_Is_host())
    {
       gettimeofday(&rend, NULL);
    }

    if (HMPI_Is_host())
    {
       gettimeofday(&gstart, NULL);
    }

    HMPI_Debug(1);

    rc = hscal_pztranc_group_heuristic_pauto_create(
              &M,
              &N,
              (double*)&ALPHA,
              &IA,
              &JA,
              DESCA,
              (double*)&BETA,
              &IC,
              &JC,
              DESCC,
              &gid,
              &MPC_NetType_pztranc,
              hscal_heuristic_fn_all
    );

    if (rc != HSCAL_SUCCESS)
    {
       return rc;
    }

    HMPI_Debug(0);

    if (HMPI_Is_host())
    {
       gettimeofday(&gend, NULL);
    }

    if (HMPI_Is_free())
    {
       HMPI_Finalize(0);
    }

    if (HMPI_Is_host())
    {
       gettimeofday(&astart, NULL);
    }

    pztranccomm = *(MPI_Comm*)HMPI_Get_comm(&gid);

    /*
     * Broadcast the number of processes to
     * all the processors
     */
    if (HMPI_Is_parent(&gid))
    {
       int nd;
       int *d;

       HMPI_Group_topology(
           &gid,
           &nd,
           &d
       );

       p = d[0];

       free(d);

       rc = MPI_Bcast(
               &p,
               1,
               MPI_INT,
               0,
               pztranccomm
       );
    }
    else
    {
       rc = MPI_Bcast(
               &p,
               1,
               MPI_INT,
               0,
               pztranccomm
       );
    }

    if (rc != MPI_SUCCESS)
    {
       printf("Problems broadcasting p\n");
       MPI_Abort(MPI_COMM_WORLD, -9);
    }

    if (HMPI_Is_parent(&gid))
    {
       int nd;
       int *d;

       HMPI_Group_topology(
           &gid,
           &nd,
           &d
       );

       q = d[1];

       free(d);

       rc = MPI_Bcast(
               &q,
               1,
               MPI_INT,
               0,
               pztranccomm
       );
    }
    else
    {
       rc = MPI_Bcast(
               &q,
               1,
               MPI_INT,
               0,
               pztranccomm
       );
    }

    if (rc != MPI_SUCCESS)
    {
       printf("Problems broadcasting q\n");
       MPI_Abort(MPI_COMM_WORLD, -9);
    }

    if (HMPI_Is_parent(&gid))
    {
       printf("Optimal p=%d, Optimal q=%d\n", p, q);
    }

    /*
     * Translate algocomm to a BLACS handle
     */
    ictxt = Csys2blacs_handle(pztranccomm);

    /*
     * Form BLACS context based on algocomm
     */
    Cblacs_gridinit(&ictxt, "r", p, q);
    Cblacs_gridinfo(ictxt, &p, &q, &myrow, &mycol);

    DESCA[DTYPE_] = BLOCK_CYCLIC_2D_INB;
    DESCA[CTXT_]  = ictxt;
    DESCA[M_]     = N;
    DESCA[N_]     = M;
    DESCA[IMB_]   = NB;
    DESCA[INB_]   = MB;
    DESCA[MB_]    = NB;
    DESCA[NB_]    = MB;
    DESCA[RSRC_]  = 0;
    DESCA[CSRC_]  = 0;

    DESCC[DTYPE_] = BLOCK_CYCLIC_2D_INB;
    DESCC[CTXT_]  = ictxt;
    DESCC[M_]     = M;
    DESCC[N_]     = N;
    DESCC[IMB_]   = MB;
    DESCC[INB_]   = NB;
    DESCC[MB_]    = MB;
    DESCC[NB_]    = NB;
    DESCC[RSRC_]  = 0;
    DESCC[CSRC_]  = 0;

    DESCA[LLD_]   = max(1, PB_Cnumroc(DESCA[M_],
                                      0,
                                      DESCA[IMB_],
                                      DESCA[MB_],
                                      myrow,
                                      DESCA[RSRC_],
                                      p
                       )
    );

    Ka            = max(1, PB_Cnumroc(JA+M-1,
                                      0,
                                      DESCA[INB_],
                                      DESCA[NB_],
                                      mycol,
                                      DESCA[CSRC_],
                                      q
                       )
    );

    DESCC[LLD_]   = max(1, PB_Cnumroc(DESCC[M_],
                                      0,
                                      DESCC[IMB_],
                                      DESCC[MB_],
                                      myrow,
                                      DESCC[RSRC_],
                                      p
                       )
    );

    Kc            = max(1, PB_Cnumroc(JC+N-1,
                                      0,
                                      DESCC[INB_],
                                      DESCC[NB_],
                                      mycol,
                                      DESCC[CSRC_],
                                      q
                       )
    );

    A = (hscaldoublecomplex*)malloc(
                  sizeof(hscaldoublecomplex)
                  *
                  (DESCA[LLD_]*Ka)
    );

    if (A == NULL)
    {
       printf("Cannot allocate A\n");
       HMPI_Finalize(-1);
    }

    C = (hscaldoublecomplex*)malloc(
                  sizeof(hscaldoublecomplex)
                  *
                  (DESCC[LLD_]*Kc)
    );

    if (C == NULL)
    {
       printf("Cannot allocate C\n");
       HMPI_Finalize(-1);
    }

    {
       int     iaseed = 100;
       hscalt_pzmatgen(&ictxt, "N", "N", &DESCA[M_],
                 &DESCA[N_], &DESCA[MB_], &DESCA[NB_], A, &DESCA[LLD_],
                 &DESCA[RSRC_], &DESCA[CSRC_], &iaseed, &c__0, &DESCA[LLD_], &c__0, &Ka,
                 &myrow, &mycol, &p, &q);
    }

    {
       int     iaseed = 200;
       hscalt_pzmatgen(&ictxt, "N", "N", &DESCC[M_],
                 &DESCC[N_], &DESCC[MB_], &DESCC[NB_], C, &DESCC[LLD_],
                 &DESCC[RSRC_], &DESCC[CSRC_], &iaseed, &c__0, &DESCC[LLD_], &c__0, &Kc,
                 &myrow, &mycol, &p, &q);
    }

    pztranc_(
      &M, &N, 
      (double*)&ALPHA,
      (double*)A, &IA, &JA, DESCA,
      (double*)&BETA,
      (double*)C, &IC, &JC, DESCC
    );

    //
    // Check the results
    if (CHECK_RESULT)
    {
       if (HMPI_Is_host())
       {
          for (i = 0; i < (DESCC[LLD_]*Kc); i++)
          {
              printf("C[%d]=(%f, %f) ", i, C[i].r, C[i].i);
          }
          printf("\n");
       }
    }

    free(A);
    free(C);

    Cblacs_gridexit(ictxt);

    if (HMPI_Is_member(&gid))
    {
       HMPI_Group_free(
           &gid
       );
    }

    if (HMPI_Is_host())
    {
       gettimeofday(&aend, NULL);
    }

    if (HMPI_Is_host())
    {
       gettimeofday(&end, NULL);
    }

    /*
     * Print Execution time
     */
    if (HMPI_Is_host())
    {
       double rtstart = rstart.tv_sec + rstart.tv_usec/1000000.;
       double rtend = rend.tv_sec + rend.tv_usec/1000000.;
       double rt_sec = rtend - rtstart;

       double gtstart = gstart.tv_sec + gstart.tv_usec/1000000.;
       double gtend = gend.tv_sec + gend.tv_usec/1000000.;
       double gt_sec = gtend - gtstart;

       double atstart = astart.tv_sec + astart.tv_usec/1000000.;
       double atend = aend.tv_sec + aend.tv_usec/1000000.;
       double at_sec = atend - atstart;

       double tstart = start.tv_sec + start.tv_usec/1000000.;
       double tend   = end.tv_sec + end.tv_usec/1000000.;
       double t_sec = tend - tstart;

       printf(
          "M=%d, N=%d, MB=%d, NB=%d, "
          "Recon time(sec)=%f, "
          "Group creation time(sec)=%f, Algorithm execution time (sec)=%f, total time(sec)=%f\n",
          M,
          N,
          MB,
          NB,
          rt_sec,
          gt_sec,
          at_sec,
          t_sec
       );
    }

    HMPI_Finalize(0);
}
